﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using ThinkGeo.MapSuite.Shapes;
using ThinkGeo.MapSuite.Wpf;
using ThinkGeoGis.Helper;

namespace ThinkGeoGis.GisCore
{
    [Serializable]
    public class DotAnimationOverylay : Overlay
    {
        private bool IsComplateDraw = true;
        [Obfuscation(Exclude = true)]
        private List<GisPoint> points;
        public List<GisPoint> Points
        {
            get
            {
                return points;
            }
        }
        public DotAnimationOverylay() : this(new List<GisPoint>())
        {
        }
        public DotAnimationOverylay(IEnumerable<GisPoint> pointshape)
        {
            base.CanRefreshRegion = true;
            base.OverlayCanvas.SetValue(Panel.ZIndexProperty, 903);
            this.points = new List<GisPoint>();
            foreach (GisPoint current in points)
            {
                this.points.Add(current);
            }
        }
        protected override void DrawCore(RectangleShape targetExtent, OverlayRefreshType overlayRefreshType)
        {
            Validators.CheckOverlayRefreshTypeIsValid(overlayRefreshType, "overlayRefreshType");
            if (overlayRefreshType == OverlayRefreshType.Pan)
            {
                IEnumerator enumerator = base.OverlayCanvas.Children.GetEnumerator();
                try
                {
                    while (enumerator.MoveNext())
                    {
                        var gisp = points.Find(s => s.ellipse.GetHashCode() == enumerator.Current.GetHashCode());
                        if (gisp != null)
                            gisp.Draw(targetExtent, base.MapArguments.ActualWidth, base.MapArguments.ActualHeight);
                    }
                    return;
                }
                finally
                {
                    IDisposable disposable = enumerator as IDisposable;
                    if (disposable != null)
                    {
                        disposable.Dispose();
                    }
                }
            }
            if (IsComplateDraw == false) return;
            ThreadPool.QueueUserWorkItem(DrawDot, targetExtent);
            
            this.OnDrawn(new DrawnOverlayEventArgs(targetExtent));
        }

        private void DrawDot(object shapeRect)
        {
            lock (this.points)
            {
                base.OverlayCanvas.Dispatcher.BeginInvoke(new Action(() =>
                {
                    IsComplateDraw = false;
                    base.OverlayCanvas.Children.Clear();
                    for (int i = 0; i < this.points.Count; i++)
                    {
                        GisPoint current = this.points[i];

                        //if (i ==0)
                        //{
                        //    Popup popup = new Popup(current.pointshape);
                        //    popup.ToolTip = "fdasffffffffffff";
                        //    popup.Content ="trest";
                        //    popup.FontSize = 10d;
                        //    popup.FontFamily = new System.Windows.Media.FontFamily("Verdana");
                        //    Canvas.SetTop(popup,500);
                        //    Canvas.SetLeft(popup, 500);
                        //    base.OverlayCanvas.Children.Add(popup);
                        //}
                        base.OverlayCanvas.Children.Add(current.ellipse);
                        current.Draw(shapeRect as RectangleShape, base.MapArguments.ActualWidth, base.MapArguments.ActualHeight);

                        Storyboard story = new Storyboard();

                        ScaleTransform scaleT = new ScaleTransform();

                        scaleT.CenterX = current.ellipse.Width / 2;
                        scaleT.CenterY = current.ellipse.Height / 2;
                        current.ellipse.RenderTransform = scaleT;

                        DependencyProperty[] propertyChain = new DependencyProperty[]{
                    Ellipse.RenderTransformProperty, ScaleTransform.ScaleXProperty,ScaleTransform.ScaleYProperty
                };
                        DoubleAnimation InDoubleAnimationX = new DoubleAnimation();
                        InDoubleAnimationX.From = 1;
                        InDoubleAnimationX.To = 1.5;
                        InDoubleAnimationX.Duration = new Duration(TimeSpan.FromSeconds(3));
                        InDoubleAnimationX.RepeatBehavior = RepeatBehavior.Forever;

                        Storyboard.SetTarget(InDoubleAnimationX, current.ellipse);
                        Storyboard.SetTargetProperty(InDoubleAnimationX, new PropertyPath("(0).(1)", propertyChain));

                        DoubleAnimation InDoubleAnimationY = new DoubleAnimation();
                        InDoubleAnimationY.From = 1;
                        InDoubleAnimationY.To = 1.5;
                        InDoubleAnimationY.Duration = new Duration(TimeSpan.FromSeconds(3));
                        InDoubleAnimationY.RepeatBehavior = RepeatBehavior.Forever;

                        Storyboard.SetTarget(InDoubleAnimationY, current.ellipse);
                        Storyboard.SetTargetProperty(InDoubleAnimationY, new PropertyPath("(0).(2)", propertyChain));

                        story.Children.Add(InDoubleAnimationX);
                        story.Children.Add(InDoubleAnimationY);

                        DoubleAnimation opAnimation = new DoubleAnimation(1, 0.55, new Duration(TimeSpan.FromSeconds(3)));
                       // opAnimation.AccelerationRatio = 0.99;
                        opAnimation.RepeatBehavior = RepeatBehavior.Forever;

                        story.Children.Add(opAnimation);
                        Storyboard.SetTarget(opAnimation, current.ellipse);
                        Storyboard.SetTargetProperty(opAnimation, new PropertyPath("Opacity"));

                        story.Begin();
                        if (i % 100 == 0)
                        {
                          //  UIEventHelper.DoUIEvents();
                        }
                    }
                    IsComplateDraw = true;
                }));
            }
        }

    }
}
