﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Globalization;
using System.Reflection;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Effects;
using System.Windows.Shapes;
using ThinkGeo.MapSuite.Serialize;
using ThinkGeo.MapSuite.Shapes;
using ThinkGeoGis.Helper;

namespace ThinkGeoGis.GisCore
{
    [DesignTimeVisible(false), ToolboxItem(false), NonSerializableBaseType]
    [Serializable]
    public class GisPath
    {
        public PointShape pointshape1;
        public PointShape pointshape2;
        public PointShape centershape;

        public Path gisPath;

        private object SRg(params object[] param)
        {
            var result = new AssemblyHandler();
            ;
            var classn = result.GetClassName("ThinkGeo.MapSuite.Wpf.dll");

            Assembly assembly = Assembly.LoadFile(AppDomain.CurrentDomain.SetupInformation.ApplicationBase + "ThinkGeo.MapSuite.Wpf.dll");
            Type type = assembly.GetType("slM=.Pxg=");
            MethodInfo medinfo = type.GetMethod("SRg=", BindingFlags.Static | BindingFlags.Public);
            object obj = medinfo.Invoke(null, param);
            return obj;
        }
        public GisPath(PointShape pt1, PointShape pt2)
        {
            gisPath = new Path();
            gisPath.Stroke = Brushes.Red;
            gisPath.StrokeThickness = 1;
            gisPath.Stretch = Stretch.Uniform;
            gisPath.ToolTip = "path";
            pointshape1 = pt1;
            pointshape2 = pt2;

            LineShape lineshape = new LineShape();
            lineshape.Vertices.Add(new Vertex(pointshape1));
            lineshape.Vertices.Add(new Vertex(pointshape2));
            centershape = lineshape.GetPointOnALine(StartingPoint.FirstPoint, 50f);
        }

        public void Draw(RectangleShape targetExtent, double drawingScreenWidth, double drawingScreenHeight)
        {
            // dynamic obj1 = SRg(new object[] { targetExtent, pointshape1.X, pointshape1.Y, drawingScreenWidth, drawingScreenHeight });
            ScreenPointF point1 = ExtentHelper.ToScreenCoordinate(targetExtent, pointshape1, (float)drawingScreenWidth, (float)drawingScreenHeight);
            ScreenPointF point2 = ExtentHelper.ToScreenCoordinate(targetExtent, centershape, (float)drawingScreenWidth, (float)drawingScreenHeight);
            ScreenPointF point3 = ExtentHelper.ToScreenCoordinate(targetExtent, pointshape2, (float)drawingScreenWidth, (float)drawingScreenHeight);

            Point pt = new Point();
            if (point1.X < point3.X)
            {
                pt.X = point1.X;
            }
            else
            {
                pt.X = point3.X;
            }
            if (point1.Y < point3.Y)
            {
                pt.Y = point1.Y;
            }
            else
            {
                pt.Y = point3.Y;
            }

            UpdateGisPathData(new Point(point1.X, point1.Y), new Point(point2.X, point2.Y), new Point(point3.X, point3.Y));
            Canvas.SetLeft(gisPath, pt.X);
            Canvas.SetTop(gisPath, pt.Y);
        }

        public void UpdateGisPathData(Point pt1, Point pt2, Point pt3)
        {
            double Angle = 0.4; //Math.Tan(60);
            var ptBeg = GetPoint3(Angle, Angle, pt1, pt3);
            //var points = Bezier.draw_bezier_curves(new Point[] { ptBeg[0], ptBeg[1], ptBeg[2] }, 3, 0.001f);
            //PathFigure pf = new PathFigure(pt1, new[] { new PolyLineSegment(points, true) }, false);
            //PathFigureCollection pfc = new PathFigureCollection();
            //pfc.Add(pf);
            //var pg = new PathGeometry();
            //pg.Figures = pfc;

            PathGeometry pg = new PathGeometry();
            PathFigureCollection figures = new PathFigureCollection();

            PathFigure pathFigure = new PathFigure();
            pathFigure.StartPoint = pt1;
            PathSegmentCollection segmentCollection = new PathSegmentCollection();
            segmentCollection.Add(new BezierSegment()
            {
                Point1 = ptBeg[0],
                Point2 = ptBeg[1],
                Point3 = ptBeg[2]
            });

            pathFigure.Segments = segmentCollection;
            figures.Add(pathFigure);

            pg.Figures = figures;
            this.gisPath.Data = pg;
        }


        public Point[] GetPoint3(double a1, double a2, Point pt1, Point pt2)
        {
            double Ag = Math.Atan2(pt2.Y - pt1.Y, pt2.X - pt1.X);
            double[] arr_A1 = { Ag - a1, Ag + a1 };
            double[] arr_A2 = { Ag + a2, Ag - a2 };

            double k = (pt2.Y - pt1.Y) / (pt2.X - pt1.X);
            double b = pt1.Y - k * pt1.X;
            int i;
            double A1, A2, kA1, kB1, kC1, kA2, kB2, kC2, rho1, rho2, x_left, x_right, y_left, y_right;
            Point[] pt3 = new Point[2];
            for (i = 0; i != 2; i++)
            {
                A1 = arr_A1[i];
                A2 = arr_A2[i];
                kA1 = Math.Cos(A1);
                kB1 = -Math.Cos(A2);
                kC1 = pt2.X - pt1.X;

                kA2 = Math.Sin(A1);
                kB2 = -Math.Sin(A2);
                kC2 = pt2.Y - pt1.Y;

                rho1 = (kB1 * kC2 - kB2 * kC1) / (kB1 * kA2 - kB2 * kA1);
                rho2 = (kA1 * kC2 - kA2 * kC1) / (kA1 * kB2 - kA2 * kB1);

                x_left = rho1 * Math.Cos(A1) + pt1.X;
                x_right = rho2 * Math.Cos(A2) + pt2.X;
                y_left = rho1 * Math.Sin(A1) + pt1.Y;
                y_right = rho2 * Math.Sin(A2) + pt2.Y;

                if (Math.Abs(x_left - x_right) < 1e-2
                && Math.Abs(y_left - y_right) < 1e-2)
                {

                    pt3[i].X = x_left;
                    pt3[i].Y = y_left;
                }
            }
            if (k> 1)
            {
                return new Point[] { pt1, pt3[0], pt2 };
            }
            else
            {
                return CheckPoint(pt1, pt2, pt3);
            }

        }

        private Point[] CheckPoint(Point pt1, Point pt2, Point[] pt)
        {
            double min = Math.Min(pt1.X, pt2.X);
            double max = Math.Min(pt1.X, pt2.X);

            Point point1 = pt[0];
            Point point2 = pt[1];

            if ((point1.X < min && point2.X < min) || (point1.X > max && point2.X > max))
            {
                if (point1.Y > point2.Y)
                {
                    return new Point[] { pt1, point1, pt2 };
                }
                else
                {
                    return new Point[] { pt1, point2, pt2 };
                }
            }
            else
            {
                if ((point1.X > min && point1.X < max) && ((point2.X < min || point2.X > max)))
                {
                    return new Point[] { pt1, point1, pt2 };
                }
                else if ((point2.X > min && point2.X < max) && ((point1.X < min || point1.X > max)))
                {
                    return new Point[] { pt1, point2, pt2 };
                }
                else
                {
                    if (point1.Y > point2.Y)
                    {
                        return new Point[] { pt1, point1, pt2 };
                    }
                    else
                    {
                        return new Point[] { pt1, point2, pt2 };
                    }
                }

            }
        }
    }
}
