﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using ThinkGeo.MapSuite;
using ThinkGeo.MapSuite.Shapes;
using ThinkGeo.MapSuite.Wpf;
using ThinkGeoGis.Helper;

namespace ThinkGeoGis.GisCore
{
    [Serializable]
    public class PathOverylay : Overlay
    {
        protected bool IsAnimationRuning = false;
        private bool IsComplateDraw = true;
        private SolidColorBrush AnimationPathBrush;
        private SolidColorBrush AnimationPathFill;
        private double AnimationPathThick;
        private string AnimationPathData;

        private double AnimationPathWidth;
        private double AnimationPathHeight;

        [Obfuscation(Exclude = true)]
        private List<GisPath> paths;
        public List<GisPath> Paths
        {
            get
            {
                return paths;
            }
        }
        public PathOverylay() : this(new Collection<GisPath>())
        {
            AnimationPathData = "M1705.06,1318.313v-89.254l-319.9-221.799l0.073-208.063c0.521-84.662-26.629-121.796-63.961-121.491c-37.332-0.305-64.482,36.829-63.961,121.491l0.073,208.063l-319.9,221.799v89.254l330.343-157.288l12.238,241.308l-134.449,92.931l0.531,42.034l175.125-42.917l175.125,42.917l0.531-42.034l-134.449-92.931l12.238-241.308L1705.06,1318.313z";
            AnimationPathBrush = Brushes.Blue;
            AnimationPathFill = Brushes.Blue;
            AnimationPathThick = 0;
            AnimationPathWidth = 15;
            AnimationPathHeight = 15;
        }

        public PathOverylay(IEnumerable<GisPath> popups)
        {
            AnimationPathData = "M1705.06,1318.313v-89.254l-319.9-221.799l0.073-208.063c0.521-84.662-26.629-121.796-63.961-121.491c-37.332-0.305-64.482,36.829-63.961,121.491l0.073,208.063l-319.9,221.799v89.254l330.343-157.288l12.238,241.308l-134.449,92.931l0.531,42.034l175.125-42.917l175.125,42.917l0.531-42.034l-134.449-92.931l12.238-241.308L1705.06,1318.313z";
            AnimationPathBrush = Brushes.Blue;
            AnimationPathFill = Brushes.Blue;
            AnimationPathThick = 0;
            AnimationPathWidth = 15;
            AnimationPathHeight = 15;
            base.CanRefreshRegion = true;
            base.OverlayCanvas.SetValue(Panel.ZIndexProperty, 902);
            this.paths = new List<GisPath>();
            foreach (GisPath current in popups)
            {
                this.paths.Add(current);
            }
        }

        protected override RectangleShape GetBoundingBoxCore()
        {
            if (this.paths.Count > 0)
            {
                MultilineShape multipointShape = new MultilineShape();
                List<Vertex> vert = new List<Vertex>();

                foreach (GisPath current in this.paths)
                {
                    vert.Clear();
                    vert.Add(new Vertex(current.pointshape1));
                    vert.Add(new Vertex(current.pointshape2));
                    multipointShape.Lines.Add(new LineShape(vert));
                }
                return multipointShape.GetBoundingBox();
            }
            return base.GetBoundingBoxCore();
        }
        protected override void DrawCore(RectangleShape targetExtent, OverlayRefreshType overlayRefreshType)
        {
            Validators.CheckOverlayRefreshTypeIsValid(overlayRefreshType, "overlayRefreshType");
            if (overlayRefreshType == OverlayRefreshType.Pan)
            {
                IEnumerator enumerator = base.OverlayCanvas.Children.GetEnumerator();
                try
                {
                    while (enumerator.MoveNext())
                    {
                        var gisp = paths.Find(s => s.gisPath.GetHashCode() == enumerator.Current.GetHashCode());
                        if (gisp != null)
                            gisp.Draw(targetExtent, base.MapArguments.ActualWidth, base.MapArguments.ActualHeight);
                    }
                    return;
                }
                finally
                {
                    IDisposable disposable = enumerator as IDisposable;
                    if (disposable != null)
                    {
                        disposable.Dispose();
                    }
                }
            }
            if (IsComplateDraw == false) return;
            ThreadPool.QueueUserWorkItem(DrawPath, targetExtent);

            this.OnDrawn(new DrawnOverlayEventArgs(targetExtent));
        }

        private void DrawPath(object rectShape)
        {
            base.OverlayCanvas.Dispatcher.BeginInvoke(new Action(()=>{
                IsComplateDraw = false;
                base.OverlayCanvas.Children.Clear();
                for (int i = 0; i < this.paths.Count; i++)
                {
                    GisPath current = this.paths[i];
                    base.OverlayCanvas.Children.Add(current.gisPath);
                    current.Draw(rectShape as RectangleShape, base.MapArguments.ActualWidth, base.MapArguments.ActualHeight);
                    if (IsAnimationRuning)
                    {
                        Storyboard story = new Storyboard();
                        Path animationPath = new Path();
                        animationPath.Stroke = AnimationPathBrush;
                        animationPath.StrokeThickness = AnimationPathThick;
                        animationPath.Fill = AnimationPathFill;
                        animationPath.Data = PathGeometry.CreateFromGeometry(Geometry.Parse(AnimationPathData));
                        animationPath.Stretch = Stretch.Uniform;
                        animationPath.Width = AnimationPathWidth;
                        animationPath.Height = AnimationPathHeight;

                        base.OverlayCanvas.Children.Add(animationPath);
                        Canvas.SetTop(animationPath, -animationPath.Width / 2);
                        Canvas.SetLeft(animationPath, -animationPath.Height / 2);

                        animationPath.RenderTransformOrigin = new Point(0.5, 0.5);

                        TransformGroup transGroup = new TransformGroup();

                        RotateTransform rotateGroup = new RotateTransform();
                        rotateGroup.Angle = 90;
                        transGroup.Children.Add(rotateGroup);

                        MatrixTransform matrix = new MatrixTransform();
                        transGroup.Children.Add(matrix);
                        animationPath.RenderTransform = transGroup;

                        NameScope.SetNameScope(base.OverlayCanvas, new NameScope());
                        base.OverlayCanvas.RegisterName("matrix" + i.ToString(), matrix);

                        MatrixAnimationUsingPath matrixAnimation = new MatrixAnimationUsingPath();
                        matrixAnimation.PathGeometry = Paths[i].gisPath.Data.GetFlattenedPathGeometry();
                        matrixAnimation.Duration = new Duration(TimeSpan.FromSeconds(6));
                        matrixAnimation.RepeatBehavior = RepeatBehavior.Forever;
                        matrixAnimation.AutoReverse = false;
                        matrixAnimation.IsOffsetCumulative = false;
                        matrixAnimation.DoesRotateWithTangent = true;//旋转
                        story.Children.Add(matrixAnimation);

                        Storyboard.SetTargetName(matrixAnimation, "matrix" + i.ToString());
                        Storyboard.SetTargetProperty(matrixAnimation, new PropertyPath(MatrixTransform.MatrixProperty));
                        story.Begin(base.OverlayCanvas);
                    }
                    if(i%100 == 0)
                    {
                        //UIEventHelper.DoUIEvents();
                    }
                }
                IsComplateDraw = true;
            }));
        }
    }
}
