﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using ThinkGeo.MapSuite;
using ThinkGeo.MapSuite.Drawing;
using ThinkGeo.MapSuite.Layers;
using ThinkGeo.MapSuite.Shapes;
using ThinkGeo.MapSuite.Styles;
using ThinkGeo.MapSuite.Wpf;
using ThinkGeoGis.GisCore;
using ThinkGeoGis.Helper;

namespace ThinkGeoGis
{
    /// <summary>
    /// MainWindow.xaml 的交互逻辑
    /// </summary>
    public partial class MainWindow : Window
    {
        private DataTable data;
        public MainWindow()
        {
            InitializeComponent();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            data = ExcelHelper.Instance.ExcelToDataTable(dataFile.Text, "mapdata");
            MapControl.MapUnit = GeographyUnit.DecimalDegree;
                       
            //ShapeFileFeatureLayer worldLayerShade = new ShapeFileFeatureLayer();
            //worldLayerShade.RequireIndex = false;
            //worldLayerShade.ShapePathFilename = @"..\..\Data\world_adm0.shp";
            //AreaStyle areaStyleShade = new AreaStyle();
            //areaStyleShade.FillSolidBrush = new GeoSolidBrush(GeoColor.FromHtml("#002B45"));
            //areaStyleShade.OutlinePen = new GeoPen(GeoColor.FromHtml("#002B45"), 1);
            //areaStyleShade.OutlinePen.DashStyle = LineDashStyle.Solid;
            //areaStyleShade.XOffsetInPixel = -4;
            //areaStyleShade.YOffsetInPixel = -4;
            //worldLayerShade.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = areaStyleShade;
            //worldLayerShade.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            ShapeFileFeatureLayer worldLayer = new ShapeFileFeatureLayer();
            worldLayer.RequireIndex = false;
            worldLayer.ShapePathFilename = @"..\..\Data\world_adm0.shp";
            AreaStyle areaStyle = new AreaStyle();
            areaStyle.FillSolidBrush = new GeoSolidBrush(GeoColor.FromHtml("#074378"));
            areaStyle.OutlinePen = new GeoPen(GeoColor.FromHtml("#0C88C1"), 1);
            areaStyle.OutlinePen.DashStyle = LineDashStyle.Solid;
            worldLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = areaStyle;
            worldLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            LayerOverlay layerOverlay = new LayerOverlay();

            var worldLayerShade = worldLayer.CloneDeep() as ShapeFileFeatureLayer;
            AreaStyle areaStyleShade = new AreaStyle();
            areaStyleShade.FillSolidBrush = new GeoSolidBrush(GeoColor.FromHtml("#002B45"));
            areaStyleShade.OutlinePen = new GeoPen(GeoColor.FromHtml("#002B45"), 1);
            areaStyleShade.OutlinePen.DashStyle = LineDashStyle.Solid;
            areaStyleShade.XOffsetInPixel = -4;
            areaStyleShade.YOffsetInPixel = -4;
            worldLayerShade.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = areaStyleShade;
            worldLayerShade.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            //layerOverlay.Layers.Add(new BackgroundLayer(new GeoSolidBrush(GeoColor.GeographicColors.ShallowOcean)));
            layerOverlay.Layers.Add("MapLayShade", worldLayerShade);
            layerOverlay.Layers.Add("MapLay", worldLayer);

            LayerOverlay selectLay = new LayerOverlay();
            InMemoryFeatureLayer highlightLayer = new InMemoryFeatureLayer();
            highlightLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.Evergreen1;
            highlightLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = TextStyles.City1("NAME");
            highlightLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            selectLay.Layers.Add("SelectMapLay", highlightLayer);

            LayerOverlay lineOverlay = new LayerOverlay();
            InMemoryFeatureLayer linelay = new InMemoryFeatureLayer();
            linelay.DrawingQuality = DrawingQuality.HighQuality;
            linelay.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = new LineStyle(new GeoPen(GeoColor.GetRandomGeoColor(RandomColorType.All), 2));
            linelay.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            lineOverlay.Layers.Add("LineLay", linelay);

            LayerOverlay pointOverlay = new LayerOverlay();
            InMemoryFeatureLayer pointlay = new InMemoryFeatureLayer();
            pointlay.DrawingQuality = DrawingQuality.HighQuality;
            pointlay.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = new PointStyle(PointSymbolType.Circle, new GeoSolidBrush(GeoColor.GetRandomGeoColor(RandomColorType.All)), 20);
            pointlay.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            pointOverlay.Layers.Add("PointLay", pointlay);
            // We need to add the layerOverlay to map.


            LayerOverlay textOverlay = new LayerOverlay();
            InMemoryFeatureLayer textlay = new InMemoryFeatureLayer();

            textOverlay.Layers.Add("TextLay", textlay);

            MapControl.Overlays.Add("MapOverlay", layerOverlay);
            MapControl.Overlays.Add("SelectMapOverlay", selectLay);
            MapControl.Overlays.Add("LineOverlay", lineOverlay);
            MapControl.Overlays.Add("TextOverlay", textOverlay);

            //SimpleMarkerOverlay mark = new SimpleMarkerOverlay();
            //mark.OverlayCanvas.MouseMove += OverlayCanvas_MouseMove1;
            //Marker mark1 = new Marker(new PointShape(116.410018, 39.912943));
            //mark1.ImageSource = null;
            //mark1.Content = new TextBlock() { Text = "test" };
            //mark1.ToolTip = "fdsafwe";
            //mark.Markers.Add(mark1);


            PathAnuimationOverylay pathOverylay = new PathAnuimationOverylay();
            MapControl.Overlays.Add("PathAnuimationOverylay", pathOverylay);

            DotAnimationOverylay dotOverlay = new DotAnimationOverylay();
            //MapControl.Overlays.Add(mark);
            MapControl.Overlays.Add("PointOverlay", pointOverlay);
            MapControl.Overlays.Add("DotAnimationOverlay", dotOverlay);
            // Set a proper extent for the Map.  
            MapControl.CurrentExtent = new RectangleShape(-180,90,180,-90);
            // We now need to call the Refresh() method of the Map control so that the Map can redraw based on the data that has been provided.
            MapControl.Refresh();
        }

        private void OverlayCanvas_MouseMove1(object sender, MouseEventArgs e)
        {
            //throw new NotImplementedException();
        }


        private void showDot_Click(object sender, RoutedEventArgs e)
        {
            var pointsFeature = GetDotPointShape();
            InMemoryFeatureLayer inMemoryLayer = (InMemoryFeatureLayer)MapControl.FindFeatureLayer("PointLay");
            inMemoryLayer.InternalFeatures.Clear();
            foreach (var feature in pointsFeature)
            {
                inMemoryLayer.InternalFeatures.Add(feature);
            }
            MapControl.Overlays["PointOverlay"].Refresh();
        }


        private void showLine_Click(object sender, RoutedEventArgs e)
        {
            InMemoryFeatureLayer inMemoryLayer = (InMemoryFeatureLayer)MapControl.FindFeatureLayer("LineLay");
            inMemoryLayer.InternalFeatures.Clear();
            for (int i = 0; i < data.Rows.Count; i++)
            {
                if (data.Rows[i][2].ToString() == "" || data.Rows[i][3].ToString() == "" || data.Rows[i][6].ToString() == "" || data.Rows[i][7].ToString() == "")
                    continue;

                LineShapeHelper lineshapeHelper = new LineShapeHelper();
                lineshapeHelper.DestinationLocation = new PointShape(double.Parse(data.Rows[i][2].ToString()), double.Parse(data.Rows[i][3].ToString()));
                lineshapeHelper.OriginLocation = new PointShape(double.Parse(data.Rows[i][6].ToString()), double.Parse(data.Rows[i][7].ToString()));
                var mutlineshape = lineshapeHelper.GetArc(2);
                foreach (var lineshape in mutlineshape.Lines)
                {
                    inMemoryLayer.InternalFeatures.Add(new Feature(lineshape));
                }
            }
            MapControl.Overlays["LineOverlay"].Refresh();
        }

        private GeoCollection<Feature> GetDotPointShape()
        {
            if (data == null)
                return null;
            GeoCollection<Feature> pointShpeList = new GeoCollection<Feature>();

            for (int i = 0; i < data.Rows.Count; i++)
            {
                if (data.Rows[i][2].ToString() != "" && data.Rows[i][3].ToString() != "")
                    pointShpeList.Add(new Feature(double.Parse(data.Rows[i][2].ToString()), double.Parse(data.Rows[i][3].ToString())));
                if (data.Rows[i][6].ToString() != "" && data.Rows[i][7].ToString() != "")
                    pointShpeList.Add(new Feature(double.Parse(data.Rows[i][6].ToString()), double.Parse(data.Rows[i][7].ToString())));
            }
            return pointShpeList;
        }

        private void showText_Click(object sender, RoutedEventArgs e)
        {

            InMemoryFeatureLayer layer = (InMemoryFeatureLayer)MapControl.FindFeatureLayer("TextLay");
            layer.Columns.Add(new FeatureSourceColumn("label"));
            for (int i = 0; i < data.Rows.Count; i++)
            {
                if (data.Rows[i][2].ToString() != "" && data.Rows[i][3].ToString() != "")
                {
                    Feature feature1 = new Feature(double.Parse(data.Rows[i][2].ToString()), double.Parse(data.Rows[i][3].ToString()));
                    feature1.ColumnValues.Add("label", data.Rows[i][1].ToString());
                    layer.InternalFeatures.Add(feature1);
                }
                if (data.Rows[i][6].ToString() != "" && data.Rows[i][7].ToString() != "")
                {
                    Feature feature1 = new Feature(double.Parse(data.Rows[i][6].ToString()), double.Parse(data.Rows[i][7].ToString()));
                    feature1.ColumnValues.Add("label", data.Rows[i][5].ToString());
                    layer.InternalFeatures.Add(feature1);
                }
            }
            layer.DrawingQuality = DrawingQuality.HighQuality;
            layer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = TextStyles.CreateSimpleTextStyle("label", null, 12, DrawingFontStyles.Underline, GeoColors.Yellow);
            layer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            MapControl.Overlays["TextOverlay"].Refresh();
        }

        private Random random = new Random();

        private PointShape GetRandomPoint(RectangleShape currentExtent)
        {
            double x = random.Next((int)currentExtent.UpperLeftPoint.X, (int)currentExtent.LowerRightPoint.X);
            double y = random.Next((int)currentExtent.LowerRightPoint.Y, (int)currentExtent.UpperLeftPoint.Y);

            PointShape pointShape = new PointShape(x, y);

            return pointShape;
        }

        private void showPathLine_Click(object sender, RoutedEventArgs e)
        {
            var textOverlay = MapControl.Overlays["PathAnuimationOverylay"] as PathAnuimationOverylay;
            textOverlay.OverlayCanvas.Children.Clear();
            textOverlay.Paths.Clear();
            for (int i = 0; i < data.Rows.Count; i++)
            {
                if (data.Rows[i][2].ToString() == "" || data.Rows[i][3].ToString() == "" || data.Rows[i][6].ToString() == "" || data.Rows[i][7].ToString() == "")
                    continue;

                GisPath gispath = new GisPath(new PointShape(double.Parse(data.Rows[i][2].ToString()), double.Parse(data.Rows[i][3].ToString())), new PointShape(double.Parse(data.Rows[i][6].ToString()), double.Parse(data.Rows[i][7].ToString())));
                textOverlay.Paths.Add(gispath);
            }
            //for (int i = 0; i < 1000; i++)
            //{
            //    GisPath gispath = new GisPath(GetRandomPoint(this.MapControl.CurrentExtent), GetRandomPoint(this.MapControl.CurrentExtent));
            //    textOverlay.Paths.Add(gispath);
            //}
            textOverlay.Refresh();
        }

        private void showPathAnimation_Click(object sender, RoutedEventArgs e)
        {
            var textOverlay = MapControl.Overlays["PathAnuimationOverylay"] as PathAnuimationOverylay;
            textOverlay.StartAnimation();
            textOverlay.Refresh();
        }


        private void MapControl_MapClick(object sender, MapClickWpfMapEventArgs e)
        {
            screentext.Text = string.Format("屏幕坐标{0},{1}", e.ScreenX.ToString(), e.ScreenY.ToString());
            worldtext.Text = string.Format("世界坐标{0},{1}", e.WorldX.ToString(), e.WorldY.ToString());

            ShapeFileFeatureLayer inMemoryLayer = (ShapeFileFeatureLayer)MapControl.FindFeatureLayer("MapLay");
            LayerOverlay selectOverlay = MapControl.Overlays["SelectMapOverlay"] as LayerOverlay;
            InMemoryFeatureLayer selectLay = (InMemoryFeatureLayer)MapControl.FindFeatureLayer("SelectMapLay");
            inMemoryLayer.Open();
            var featureSelect = inMemoryLayer.QueryTools.GetFeaturesContaining(e.WorldLocation, new string[] { "NAME" });
            inMemoryLayer.Close();
            selectLay.InternalFeatures.Clear();
            foreach (var feature in featureSelect)
            {
                selectLay.InternalFeatures.Add(feature);
            }
            selectOverlay.Refresh();
        }

        private void showDotAnimation_Click(object sender, RoutedEventArgs e)
        {
            var dotAnimationOverlay = MapControl.Overlays["DotAnimationOverlay"] as DotAnimationOverylay;
            dotAnimationOverlay.OverlayCanvas.Children.Clear();
            dotAnimationOverlay.Points.Clear();
            for (int i = 0; i < data.Rows.Count; i++)
            {
                if (data.Rows[i][2].ToString() != "" && data.Rows[i][3].ToString() != "")
                {
                    var pointx = dotAnimationOverlay.Points.Find(s => s.pointshape.X == double.Parse(data.Rows[i][2].ToString()));
                    var pointy = dotAnimationOverlay.Points.Find(s => s.pointshape.Y == double.Parse(data.Rows[i][3].ToString()));
                    if (pointx == null || pointy == null)
                    {
                        dotAnimationOverlay.Points.Add(new GisPoint(new PointShape(double.Parse(data.Rows[i][2].ToString()), double.Parse(data.Rows[i][3].ToString()))));
                    }
                }
                if (data.Rows[i][6].ToString() != "" && data.Rows[i][7].ToString() != "")
                {
                    var pointx = dotAnimationOverlay.Points.Find(s => s.pointshape.X == double.Parse(data.Rows[i][6].ToString()));
                    var pointy = dotAnimationOverlay.Points.Find(s => s.pointshape.Y == double.Parse(data.Rows[i][7].ToString()));
                    if (pointx == null || pointy == null)
                    {
                        dotAnimationOverlay.Points.Add(new GisPoint(new PointShape(double.Parse(data.Rows[i][6].ToString()), double.Parse(data.Rows[i][7].ToString()))));
                    }
                }
            }
            //for (int i = 0; i < 1000; i++)
            //{
            //    GisPoint gispath = new GisPoint(GetRandomPoint(this.MapControl.CurrentExtent));
            //    dotAnimationOverlay.Points.Add(gispath);
            //}
            dotAnimationOverlay.Refresh();
        }
    }
}
